#include "editor.h"
#include "bridge.h"
#include "../common/sources/system/cgraphics2d.h"

CEditor  editor;

//---------------------------------------------------------------------------------------------
CEditor::CEditor()
{

}

//---------------------------------------------------------------------------------------------
EWallSide   CEditor::getWallSide(CVec2 localPos)
{
    bool side1 = (localPos.y < localPos.x);
    bool side2 = (localPos.y < TILE_SIZE - 1 - localPos.x);

    if (side1 == true && side2 == true)
        return eWallSideUp;

    if (side1 == true && side2 == false)
        return eWallSideRight;

    if (side1 == false && side2 == true)
        return eWallSideLeft;

    return eWallSideDown;
}

//---------------------------------------------------------------------------------------------
QImage CEditor::displayMap(CMap& map, CVec2 mousePos, int tabIndex)
{
    QImage image(map.mSize.x * TILE_SIZE,
                 map.mSize.y * TILE_SIZE,
                 QImage::Format_RGBA8888);

    image.fill(QColor(0, 0, 0));

    QImage  tileGfx("tiles/Tile.png");
    QImage  wallGfx("tiles/Wall.png");

    if (map.isLoaded() == true)
    {
        // draw tiles
        for (int y = 0; y < map.mSize.y; ++y)
            for (int x = 0; x < map.mSize.x; ++x)
            {
                CVec2   pos(x, y);
                CVec2   pos2 = pos * TILE_SIZE;

                if (map.getTile(pos)->mType != 0)
                    graph2D.drawImage(&image, pos2, tileGfx);
            }

        // draw walls
        for (int y = 0; y < map.mSize.y; ++y)
            for (int x = 0; x < map.mSize.x; ++x)
            {
                CVec2   pos(x, y);
                CVec2   pos2 = pos * TILE_SIZE;
                CTile*  tile = map.getTile(pos);

                if (tile->mWalls[eWallSideUp].mType != 0)
                    graph2D.drawImage(&image, pos2, wallGfx);

                if (tile->mWalls[eWallSideDown].mType != 0)
                    graph2D.drawImage(&image, pos2, wallGfx, 180);

                if (tile->mWalls[eWallSideLeft].mType != 0)
                    graph2D.drawImage(&image, pos2, wallGfx, -90);

                if (tile->mWalls[eWallSideRight].mType != 0)
                    graph2D.drawImage(&image, pos2, wallGfx, 90);
            }
    }

    CVec2   pos = mousePos / TILE_SIZE;
    CVec2   pos2 = pos * TILE_SIZE;

    // mouse position
    if (mousePos.x != -1)
    {
        if (tabIndex == 1)
        {
            // Tile
            CRect   r(CVec2(pos2), CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
            graph2D.rect(&image, r, QColor(0, 255, 0), false);
        }
        else if (tabIndex == 2)
        {
            // Wall
            CRect   r;

            CVec2   localPos = mousePos - pos2;
            EWallSide   side = getWallSide(localPos);

            switch (side)
            {
                case eWallSideUp:
                    r = CRect(pos2, CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE / 4 - 1));
                    break;

                case eWallSideDown:
                    r = CRect(CVec2(pos2.x, pos2.y + (TILE_SIZE * 3) / 4),
                              CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
                    break;

                case eWallSideLeft:
                    r = CRect(pos2, CVec2(pos2.x + TILE_SIZE / 4 - 1, pos2.y + TILE_SIZE - 1));
                    break;

                case eWallSideRight:
                    r = CRect(CVec2(pos2.x + (TILE_SIZE * 3) / 4, pos2.y),
                              CVec2(pos2.x + TILE_SIZE - 1, pos2.y + TILE_SIZE - 1));
                    break;

                default:
                    break;
            }

            graph2D.rect(&image, r, QColor(0, 255, 0), false);
        }
        else if (tabIndex == 3)
        {
            // Objects
            CRect   r;

            CVec2   localPos = mousePos;
            localPos /= TILE_SIZE / 2;
            localPos *= TILE_SIZE / 2;

            r = CRect(localPos, CVec2(localPos.x + TILE_SIZE / 2 - 1, localPos.y + TILE_SIZE / 2 - 1));
            graph2D.rect(&image, r, QColor(0, 255, 0), false);
        }
    }

    return image;
}

//---------------------------------------------------------------------------------------------
void    CEditor::useTool()
{
    toolDraw();
}

//---------------------------------------------------------------------------------------------
void    CEditor::toolDraw()
{
    CVec2   tilePos = bridge.mMousePos / TILE_SIZE;

    if (bridge.mTabIndex == 1)
    {
        // tiles
        if (bridge.rightPressed == true)
            map.setTile(tilePos, 0);
        else if (bridge.leftPressed == true)
            map.setTile(tilePos, 1);
    }
    else if (bridge.mTabIndex == 2)
    {
        // walls
        CVec2   pos = tilePos * TILE_SIZE;
        CVec2   localPos = bridge.mMousePos - pos;
        EWallSide   side = getWallSide(localPos);

        if (bridge.rightPressed == true)
            map.setWall(tilePos, side, 0);
        else if (bridge.leftPressed == true)
            map.setWall(tilePos, side, 1);
    }
}
